@@ -1,7 +1,7 @@
 /*
  * (SLIK) SimpLIstic sKin functions
  * (C) 2004 John Ellis
- * Copyright (C) 2008 - 2010 The Geeqie Team
+ * Copyright (C) 2008 - 2012 The Geeqie Team
  *
  * Author: John Ellis
  *
@@ -551,7 +551,11 @@ static void pref_link_sensitivity_cb(GtkWidget *watch, GtkStateType prev_state,
 {
 	GtkWidget *widget = data;
 
+#if GTK_CHECK_VERSION(2,20,0)
+	gtk_widget_set_sensitive(widget, gtk_widget_is_sensitive(watch));
+#else
 	gtk_widget_set_sensitive(widget, GTK_WIDGET_IS_SENSITIVE(watch));
+#endif
 }
 
 void pref_link_sensitivity(GtkWidget *widget, GtkWidget *watch)
@@ -718,6 +722,20 @@ GtkWidget *pref_table_spin(GtkWidget *table, gint column, gint row,
 	return spin;
 }
 
+GtkWidget *pref_table_spin_new_int(GtkWidget *table, gint column, gint row,
+				   const gchar *text, const gchar *suffix,
+				   gint min, gint max, gint step,
+				   gint value, gint *value_var)
+{
+	*value_var = value;
+	return pref_table_spin(table, column, row,
+			       text, suffix,
+			       (gdouble)min, (gdouble)max, (gdouble)step, 0,
+			       value,
+			       G_CALLBACK(pref_spin_int_cb), value_var);
+}
+
+
 #if ! GTK_CHECK_VERSION(2,12,0)
 
 static void pref_toolbar_destroy_cb(GtkWidget *widget, gpointer data)
@@ -876,7 +894,11 @@ static void date_selection_popup_hide(DateSelection *ds)
 {
 	if (!ds->window) return;
 
+#if GTK_CHECK_VERSION(2,20,0)
+	if (gtk_widget_has_grab(ds->window))
+#else
 	if (GTK_WIDGET_HAS_GRAB(ds->window))
+#endif
 		{
 		gtk_grab_remove(ds->window);
 		gdk_keyboard_ungrab(GDK_CURRENT_TIME);
@@ -1289,7 +1311,11 @@ static gboolean sizer_release_cb(GtkWidget *widget, GdkEventButton *bevent, gpoi
 
 	if (bevent->button != MOUSE_BUTTON_LEFT) return FALSE;
 
+#if GTK_CHECK_VERSION(2,20,0)
+	if (gdk_pointer_is_grabbed() && gtk_widget_has_grab(sd->sizer))
+#else
 	if (gdk_pointer_is_grabbed() && GTK_WIDGET_HAS_GRAB(sd->sizer))
+#endif
 		{
 		gtk_grab_remove(sd->sizer);
 		gdk_pointer_ungrab(bevent->time);
